using System.Drawing.Imaging;
using Dalamud.Configuration;
using Scriban;
using Scriban.Parsing;

namespace Screenie;

[Serializable]
public class Configuration : IPluginConfiguration {
    public int Version { get; set; } = 1;

    public string SaveDirectory = Path.Join(Environment.GetFolderPath(Environment.SpecialFolder.MyPictures), "Screenie");
    public Format SaveFormat = Format.Png;
    public int SaveFormatData = 90;
    public string SaveFileNameFormat = """
                                       {{ captured_at_local | date.to_string '%Y/%m/[%H.%M.%S]' }}
                                       {{ active_character.name }}
                                       -
                                       {{ location }}
                                       {{- if area }}
                                       ({{ area }})
                                       {{- end -}}
                                       """;

    private int _templateHashCode;
    private Template? _template;

    internal Template SaveFileNameTemplate {
        get {
            var currentHash = this.SaveFileNameFormat.GetHashCode();
            if (currentHash == this._templateHashCode && this._template != null) {
                return this._template;
            }

            this._templateHashCode = currentHash;
            this._template = Template.Parse(
                this.SaveFileNameFormat,
                parserOptions: new ParserOptions {
                    LiquidFunctionsToScriban = true,
                }
            );

            return this._template;
        }
    }
}

public enum Format {
    Png,
    WebpLossless,
    WebpLossy,
    Jpg,
}

public static class FormatExt {
    public static ImageFormat? ToImageFormat(this Format format) {
        return format switch {
            Format.Png => ImageFormat.Png,
            Format.WebpLossless => null,
            Format.WebpLossy => null,
            Format.Jpg => ImageFormat.Jpeg,
            _ => throw new ArgumentOutOfRangeException(nameof(format), format, null),
        };
    }
}
