﻿using Dalamud.IoC;
using Dalamud.Plugin;
using Dalamud.Plugin.Services;
using Screenie.Ui;

namespace Screenie;

public class Plugin : IDalamudPlugin {
    internal const string Name = "Screenie";

    [PluginService]
    internal static IPluginLog Log { get; private set; }

    [PluginService]
    internal DalamudPluginInterface Interface { get; init; }

    [PluginService]
    internal IChatGui ChatGui { get; init; }

    [PluginService]
    internal IClientState ClientState { get; init; }

    [PluginService]
    internal ICommandManager CommandManager { get; init; }

    [PluginService]
    internal IDataManager DataManager { get; init; }

    [PluginService]
    internal IFramework Framework { get; init; }

    [PluginService]
    internal IGameGui GameGui { get; init; }

    [PluginService]
    internal IObjectTable ObjectTable { get; init; }

    internal Configuration Config { get; }
    internal LinkHandlers LinkHandlers { get; }
    internal PluginUi Ui { get; }

    private Command Command { get; }

    public Plugin() {
        this.Config = this.Interface!.GetPluginConfig() as Configuration ?? new Configuration();
        this.LinkHandlers = new LinkHandlers(this);
        this.Ui = new PluginUi(this);
        this.Command = new Command(this);
    }

    public void Dispose() {
        this.Command.Dispose();
        this.Ui.Dispose();
        this.LinkHandlers.Dispose();
    }

    internal void SaveConfig() {
        this.Interface.SavePluginConfig(this.Config);
    }
}
